/*
 * Copyright (c) 2016-2017 VMware, Inc. All Rights Reserved.
 * This software is released under MIT license.
 * The full license information can be found in LICENSE in the root directory of this project.
 */
import { Directive, TemplateRef, ViewContainerRef, Input, Output, EventEmitter, Inject } from "@angular/core";
import { IF_ACTIVE_ID, IfActiveService } from "./if-active.service";
/**********
 *
 * @class IfActiveDirective
 *
 * @description
 * A structural directive that controls whether or not the associated TemplateRef is instantiated or not.
 * It makes use of a Component instance level service: IfActiveService to maintain state between itself and
 * the component using it in the component template.
 *
 */
var IfActiveDirective = (function () {
    function IfActiveDirective(ifActiveService, id, template, container) {
        var _this = this;
        this.ifActiveService = ifActiveService;
        this.id = id;
        this.template = template;
        this.container = container;
        this.wasActive = false;
        /**********
         * @property activeChange
         *
         * @description
         * An event emitter that emits when the active property is set to allow for 2way binding when the directive is
         * used with de-structured / de-sugared syntax.
         *
         * @type {EventEmitter<any>}
         */
        this.activeChange = new EventEmitter(false);
        this.updateView(ifActiveService.current === this.id);
        this.subscription = this.ifActiveService.currentChange.subscribe(function (newId) {
            var isNowActive = newId === _this.id;
            // only emit if the new active state is changed since last time.
            if (isNowActive !== _this.wasActive) {
                _this.updateView(isNowActive);
                _this.activeChange.emit(isNowActive);
                _this.wasActive = isNowActive;
            }
        });
    }
    Object.defineProperty(IfActiveDirective.prototype, "active", {
        /********
         * @function active
         *
         * @description
         * A getter that returns the current IfActiveService.active value.
         * @returns {any}
         */
        get: function () {
            return this.ifActiveService.current === this.id;
        },
        /*********
         * @function active
         *
         * @description
         * A setter that updates IfActiveService.active with value.
         *
         * @param value
         */
        set: function (value) {
            if (value) {
                this.ifActiveService.current = this.id;
            }
        },
        enumerable: true,
        configurable: true
    });
    /*********
     * @function updateView
     *
     * @description
     * Function that takes a any value and either created an embedded view for the associated ViewContainerRef or,
     * Clears all views from the ViewContainerRef
     * @param value
     */
    IfActiveDirective.prototype.updateView = function (value) {
        if (value) {
            this.container.createEmbeddedView(this.template);
        }
        else {
            this.container.clear();
        }
    };
    IfActiveDirective.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    return IfActiveDirective;
}());
export { IfActiveDirective };
IfActiveDirective.decorators = [
    { type: Directive, args: [{
                selector: "[clrIfActive]"
            },] },
];
/** @nocollapse */
IfActiveDirective.ctorParameters = function () { return [
    { type: IfActiveService, },
    { type: undefined, decorators: [{ type: Inject, args: [IF_ACTIVE_ID,] },] },
    { type: TemplateRef, },
    { type: ViewContainerRef, },
]; };
IfActiveDirective.propDecorators = {
    'active': [{ type: Input, args: ["clrIfActive",] },],
    'activeChange': [{ type: Output, args: ["clrIfActiveChange",] },],
};
//# sourceMappingURL=if-active.directive.js.map